@@ -1,4 +1,4 @@
-# Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -29,7 +29,8 @@
 """Implementation of the CRUD database objects."""
 
 from .dbdoc import DbDoc
-from .errors import ProgrammingError
+from .errorcode import ER_NO_SUCH_TABLE
+from .errors import OperationalError, ProgrammingError
 from .helpers import deprecated, escape, quote_identifier
 from .statement import (FindStatement, AddStatement, RemoveStatement,
                         ModifyStatement, SelectStatement, InsertStatement,
@@ -314,7 +315,9 @@ class Collection(DatabaseObject):
             condition (Optional[str]): The string with the filter expression of
                                        the documents to be retrieved.
         """
-        return FindStatement(self, condition)
+        stmt = FindStatement(self, condition)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def add(self, *values):
         """Adds a list of documents to a collection.
@@ -341,7 +344,9 @@ class Collection(DatabaseObject):
         .. versionchanged:: 8.0.12
            The ``condition`` parameter is now mandatory.
         """
-        return RemoveStatement(self, condition)
+        stmt = RemoveStatement(self, condition)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def modify(self, condition):
         """Modifies documents based on the ``condition``.
@@ -356,7 +361,9 @@ class Collection(DatabaseObject):
         .. versionchanged:: 8.0.12
            The ``condition`` parameter is now mandatory.
         """
-        return ModifyStatement(self, condition)
+        stmt = ModifyStatement(self, condition)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def count(self):
         """Counts the documents in the collection.
@@ -366,7 +373,15 @@ class Collection(DatabaseObject):
         """
         sql = _COUNT_QUERY.format(quote_identifier(self._schema.name),
                                   quote_identifier(self._name))
-        return self._connection.execute_sql_scalar(sql)
+        try:
+            res = self._connection.execute_sql_scalar(sql)
+        except OperationalError as err:
+            if err.errno == ER_NO_SUCH_TABLE:
+                raise OperationalError(
+                    "Collection '{}' does not exist in schema '{}'"
+                    "".format(self._name, self._schema.name))
+            raise
+        return res
 
     def create_index(self, index_name, fields_desc):
         """Creates a collection index.
@@ -380,6 +395,7 @@ class Collection(DatabaseObject):
                                    {"fields": [{"field": member_path,
                                                 "type": member_type,
                                                 "required": member_required,
+                                                "array": array,
                                                 "collation": collation,
                                                 "options": options,
                                                 "srid": srid},
@@ -407,7 +423,7 @@ class Collection(DatabaseObject):
 
         Args:
             doc_id (str): Document ID
-            doc (DbDoc/dict): New Document
+            doc (:class:`mysqlx.DbDoc` or `dict`): New Document
         """
         return self.modify("_id = :id").set("$", doc) \
                    .bind("id", doc_id).execute()
@@ -418,7 +434,7 @@ class Collection(DatabaseObject):
 
         Args:
             doc_id (str): Document ID
-            doc (DbDoc/dict): New Document
+            doc (:class:`mysqlx.DbDoc` or dict): New Document
         """
         if not isinstance(doc, DbDoc):
             doc = DbDoc(doc)
@@ -429,8 +445,14 @@ class Collection(DatabaseObject):
 
         Args:
             doc_id (str): Document ID
+
+        Returns:
+            mysqlx.DbDoc: The Document matching the Document ID.
         """
-        return self.find("_id = :id").bind("id", doc_id).execute().fetch_one()
+        result = self.find("_id = :id").bind("id", doc_id).execute()
+        doc = result.fetch_one()
+        self._connection.fetch_active_result()
+        return doc
 
     def remove_one(self, doc_id):
         """Removes a Document matching the Document ID.
@@ -474,7 +496,9 @@ class Table(DatabaseObject):
         Returns:
             mysqlx.SelectStatement: SelectStatement object
         """
-        return SelectStatement(self, *fields)
+        stmt = SelectStatement(self, *fields)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def insert(self, *fields):
         """Creates a new :class:`mysqlx.InsertStatement` object.
@@ -485,7 +509,9 @@ class Table(DatabaseObject):
         Returns:
             mysqlx.InsertStatement: InsertStatement object
         """
-        return InsertStatement(self, *fields)
+        stmt = InsertStatement(self, *fields)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def update(self):
         """Creates a new :class:`mysqlx.UpdateStatement` object.
@@ -493,7 +519,9 @@ class Table(DatabaseObject):
         Returns:
             mysqlx.UpdateStatement: UpdateStatement object
         """
-        return UpdateStatement(self)
+        stmt = UpdateStatement(self)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def delete(self):
         """Creates a new :class:`mysqlx.DeleteStatement` object.
@@ -504,7 +532,9 @@ class Table(DatabaseObject):
         .. versionchanged:: 8.0.12
            The ``condition`` parameter was removed.
         """
-        return DeleteStatement(self)
+        stmt = DeleteStatement(self)
+        stmt.stmt_id = self._connection.get_next_statement_id()
+        return stmt
 
     def count(self):
         """Counts the rows in the table.
@@ -514,7 +544,15 @@ class Table(DatabaseObject):
         """
         sql = _COUNT_QUERY.format(quote_identifier(self._schema.name),
                                   quote_identifier(self._name))
-        return self._connection.execute_sql_scalar(sql)
+        try:
+            res = self._connection.execute_sql_scalar(sql)
+        except OperationalError as err:
+            if err.errno == ER_NO_SUCH_TABLE:
+                raise OperationalError(
+                    "Table '{}' does not exist in schema '{}'"
+                    "".format(self._name, self._schema.name))
+            raise
+        return res
 
     def is_view(self):
         """Determine if the underlying object is a view or not.
